package cn.iocoder.yudao.module.customer.dal.mysql.customerApproval;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQuery;
import cn.iocoder.yudao.framework.mybatis.core.mapper.AbstractMapper;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerApproval.CustomerApprovalDO;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import org.apache.ibatis.annotations.Mapper;
import cn.iocoder.yudao.module.customer.vo.customerApproval.*;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 客户相关审核详情 Mapper
 * @author 系统管理员
 */
@Mapper
public interface CustomerApprovalMapper extends AbstractMapper<CustomerApprovalDO> {
    @Override
    default PageResult<CustomerApprovalDO> selectPage(PageVO page, Object object) {
        if (object instanceof CustomerApprovalQueryVO) {
            CustomerApprovalQueryVO vo = (CustomerApprovalQueryVO)object;
            return selectPage(page, new LambdaQuery<CustomerApprovalDO>()
                .eqIfPresent(CustomerApprovalDO::getCustomerId, vo.getCustomerId())
                .eqIfPresent(CustomerApprovalDO::getFormId, vo.getFormId())
                .eqIfPresent(CustomerApprovalDO::getType, vo.getType())
                .eqIfPresent(CustomerApprovalDO::getFollowUpSalesmanId, vo.getFollowUpSalesmanId())
                .eqIfPresent(CustomerApprovalDO::getDetails, vo.getDetails())
                .eqIfPresent(CustomerApprovalDO::getStatus, vo.getStatus())
                .eqIfPresent(CustomerApprovalDO::getProcessingResults, vo.getProcessingResults())
                .betweenIfPresent(CustomerApprovalDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(CustomerApprovalDO::getFinishBy, vo.getFinishBy())
                .betweenIfPresent(CustomerApprovalDO::getFinishTime, vo.getBeginFinishTime(), vo.getEndFinishTime())
                    .orderByDesc(CustomerApprovalDO::getApprovalId));
        }

        return null;
    }

    @Override
    default List<CustomerApprovalDO> selectList(Object object) {
        if (object instanceof CustomerApprovalQueryVO) {
            CustomerApprovalQueryVO vo = (CustomerApprovalQueryVO)object;
            return selectList(new LambdaQuery<CustomerApprovalDO>()
                .eqIfPresent(CustomerApprovalDO::getCustomerId, vo.getCustomerId())
                .eqIfPresent(CustomerApprovalDO::getFormId, vo.getFormId())
                .eqIfPresent(CustomerApprovalDO::getType, vo.getType())
                .eqIfPresent(CustomerApprovalDO::getFollowUpSalesmanId, vo.getFollowUpSalesmanId())
                .eqIfPresent(CustomerApprovalDO::getDetails, vo.getDetails())
                .eqIfPresent(CustomerApprovalDO::getStatus, vo.getStatus())
                .eqIfPresent(CustomerApprovalDO::getProcessingResults, vo.getProcessingResults())
                .betweenIfPresent(CustomerApprovalDO::getCreateTime, vo.getBeginCreateTime(), vo.getEndCreateTime())
                .eqIfPresent(CustomerApprovalDO::getFinishBy, vo.getFinishBy())
                .betweenIfPresent(CustomerApprovalDO::getFinishTime, vo.getBeginFinishTime(), vo.getEndFinishTime())
                    .orderByDesc(CustomerApprovalDO::getApprovalId));
        }

        return null;
    }

    @Select({
        "SELECT * FROM ecw_customer_approval WHERE deleted=0 ",
        "AND type = 2 AND status = 1 ",
        "AND details is not null ",
        "AND details ->> '$.customerHandoverDetailDtoList' is not null ",
        "and JSON_EXTRACT(details, '$.customerHandoverDetailDtoList') like concat('%\"customerId\": ', concat(#{customerId},',%'))",
        "ORDER BY approval_id DESC limit 1"
    })
    CustomerApprovalDO getCustomerHandoverApprovalByCustomerId(@Param("customerId") Long customerId);



    @Select({
            "SELECT * FROM ecw_customer_approval WHERE deleted=0 ",
            "AND status = 1 ",
            "AND ( ",
            "(type = 2 AND details is not null  AND details ->> '$.customerHandoverDetailDtoList' is not null and JSON_EXTRACT(details, '$.customerHandoverDetailDtoList') like concat('%\"customerId\": ', concat(#{customerId},',%')))",
            " or (customer_id = #{customerId}) ",
            ") ",
            "ORDER BY approval_id DESC"
    })
    List<CustomerApprovalDO> selectInApprovalByCustomerId(@Param("customerId") Long customerId);
}
