package cn.iocoder.yudao.module.customer.dal.mysql.customerContacts;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dto.CustomerContactsDto;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsPageReqVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.util.Collection;
import java.util.List;

/**
 * 客户联系人 Mapper
 *
 * @author 系统管理员
 */
@Mapper
public interface CustomerContactsMapper extends BaseMapperX<CustomerContactsDO> {

    default PageResult<CustomerContactsDO> selectPage(CustomerContactsPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<CustomerContactsDO>()
                .likeIfPresent(CustomerContactsDO::getUsername, reqVO.getUsername())
                .eqIfPresent(CustomerContactsDO::getAreaCode, reqVO.getAreaCode())
                .eqIfPresent(CustomerContactsDO::getPhoneNew, reqVO.getPhoneNew())
                .betweenIfPresent(CustomerContactsDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(CustomerContactsDO::getId));
    }

    default List<CustomerContactsDO> selectList(CustomerContactsExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<CustomerContactsDO>()
                .likeIfPresent(CustomerContactsDO::getUsername, reqVO.getUsername())
                .eqIfPresent(CustomerContactsDO::getAreaCode, reqVO.getAreaCode())
                .eqIfPresent(CustomerContactsDO::getPhoneNew, reqVO.getPhoneNew())
                .betweenIfPresent(CustomerContactsDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(CustomerContactsDO::getId));
    }

    default List<CustomerContactsDO> selectList(CustomerContactsDO reqVO) {
        return selectList(new LambdaQueryWrapperX<CustomerContactsDO>()
                .eqIfPresent(CustomerContactsDO::getCustomerId, reqVO.getCustomerId())
                .eqIfPresent(CustomerContactsDO::getDepartment, reqVO.getDepartment())
                .eqIfPresent(CustomerContactsDO::getPosition, reqVO.getPosition())
                .likeIfPresent(CustomerContactsDO::getName, reqVO.getName())
                .eqIfPresent(CustomerContactsDO::getSocial, reqVO.getSocial())
                .eqIfPresent(CustomerContactsDO::getSocialNumber, reqVO.getSocialNumber())
                .eqIfPresent(CustomerContactsDO::getEmail, reqVO.getEmail())
                .eqIfPresent(CustomerContactsDO::getIsDefault, reqVO.getIsDefault())
                .eqIfPresent(CustomerContactsDO::getUserid, reqVO.getUserid())
                .likeIfPresent(CustomerContactsDO::getUsername, reqVO.getUsername())
                .eqIfPresent(CustomerContactsDO::getAreaCode, reqVO.getAreaCode())
                .eqIfPresent(CustomerContactsDO::getPhoneNew, reqVO.getPhoneNew())
                .eqIfPresent(CustomerContactsDO::getIsAuto, reqVO.getIsAuto())
                .orderByDesc(CustomerContactsDO::getId));
    }

    @ResultType(CustomerContactsDto.class)
    @Select({
            "<script>",
            "select",
            "c.*, ",
            "c.id as customer_id, ",
            "c.name as customer_name, ",
            "c.name_en as customer_name_en, ",
            "c.number as customer_number, ",
            "c.default_pay as default_pay, ",
            "c.no_consignee as no_consignee, ",
            "cc.* , ",
            "cc.id as customer_contacts_id, ",
            "cc.name as contacts_name, ",
            "cc.name_en as contacts_name_en ",
            "from ecw_customer_contacts cc ",
            "left join ecw_customer c ",
            "on cc.customer_id = c.id ",
            "where ",
            "cc.deleted = 0 and cc.customer_id is not null ",
            "<when test = 'searchKey != null and searchKey != \"\" '>",
            "and CONCAT(IFNULL(cc.`name`,''),IFNULL(cc.`name_en`,''),IFNULL(cc.`phone_new`,''),IFNULL(cc.`email`,''),IFNULL(c.`company`,''),IFNULL(c.`company_en`,'')) like concat('%', concat(#{searchKey}, '%'))",
            "</when>",
            "<when test = 'phone != null and phone != \"\" '>",
            "and IFNULL(cc.`phone_new`,'') like concat('%', concat(#{phone}, '%'))",
            "</when>",
            "<when test = 'type != null and type != \"\" '>",
            "and FIND_IN_SET(#{type}, c.type) ",
            "</when>",
            "<when test = 'ids != null and ids.size()>0'>",
            "AND cc.`id` in ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>#{id}</foreach>",
            "</when>",
            "<when test = 'customerId != null'>",
            "and c.id = #{customerId}",
            "</when>",
            "order by cc.update_time desc ",
            "</script>"
    })
    IPage<CustomerContactsDto> selectBySearchKey(IPage page,
                                                 @Param("searchKey") String searchKey,
                                                 @Param("phone") String phone,
                                                 @Param("type") String type,
                                                 @Param("ids") Collection<Long> ids,
                                                 @Param("customerId") Long customerId);


    @ResultType(CustomerContactsDto.class)
    @Select({
            "<script>",
            "select",
            "c.*, ",
            "c.id as customer_id, ",
            "c.name as customer_name, ",
            "c.name_en as customer_name_en, ",
            "c.number as customer_number, ",
            "c.default_pay as default_pay, ",
            "c.no_consignee as no_consignee, ",
            "cc.* , ",
            "cc.id as customer_contacts_id, ",
            "cc.name as contacts_name, ",
            "cc.name_en as contacts_name_en ",
            "from ecw_customer_contacts cc ",
            "left join ecw_customer c ",
            "on cc.customer_id = c.id ",
            "where ",
            "cc.deleted = 0 and cc.customer_id is not null ",
            "<when test = 'searchKey != null and searchKey != \"\" '>",
            "and CONCAT(IFNULL(cc.`name`,''),IFNULL(cc.`name_en`,''),IFNULL(cc.`phone_new`,''),IFNULL(cc.`email`,''),IFNULL(c.`company`,''),IFNULL(c.`company_en`,'')) like concat('%', concat(#{searchKey}, '%'))",
            "</when>",
            "<when test = 'type != null and type != \"\" '>",
            "and FIND_IN_SET(#{type}, c.type) ",
            "</when>",
            "<when test = 'ids != null and ids.size()>0'>",
            "AND cc.`id` in ",
            "<foreach item='id' index='index' collection='ids' open='(' separator=',' close=')'>#{id}</foreach>",
            "</when>",
            "<when test = 'customerId != null'>",
            "and c.id = #{customerId}",
            "</when>",
            "<when test = 'UserId != null'>",
            "and c.customer_service = #{UserId} and is_in_open_sea=0",
            "</when>",
            "order by cc.update_time desc ",
            "</script>"
    })
        //过滤客户经理函数
    IPage<CustomerContactsDto> selectBySearchKey2(IPage page,
                                                  @Param("searchKey") String searchKey,
                                                  @Param("phone") String phone,
                                                  @Param("type") String type,
                                                  @Param("ids") Collection<Long> ids,
                                                  @Param("customerId") Long customerId,
                                                  @Param("UserId") Long UserId);
}
