package cn.iocoder.yudao.module.customer.service.api;

import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.dict.core.util.DictFrameworkUtils;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.service.customer.CustomerService;
import cn.iocoder.yudao.module.customer.service.customerContacts.CustomerContactsService;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.vo.CustomerCreateReqVO;
import cn.iocoder.yudao.module.ecw.api.customer.CustomerApi;
import cn.iocoder.yudao.module.ecw.dal.dataobject.country.CountryDO;
import cn.iocoder.yudao.module.ecw.enums.CustomerCreateFromEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerStatusEnum;
import cn.iocoder.yudao.module.ecw.service.country.CountryService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;

@Service
@Validated
public class CustomerApiImpl implements CustomerApi {

    @Resource
    private CustomerService customerService;

    @Resource
    private CustomerContactsService customerContactsService;

    @Resource
    private CountryService countryService;

    public void associateCustomerAuto(String areaCode, String phone, Long memberUserId, String newName, String memberCode) {

        //先查询此用户是否有关联老客户
        CustomerContactsDO customerContactsDO = customerContactsService.getUniqueOneAndValidate(areaCode, phone, null);
        if (customerContactsDO != null) {
            //若有关联，直接更新userId
            customerContactsDO.setUserid(memberUserId);

            customerContactsService.updateById(customerContactsDO);
        } else {

            CustomerCreateReqVO customerCreateReqVO = new CustomerCreateReqVO();
            //初始化名字为手机号
            customerCreateReqVO.setName(newName);
            //普通
            customerCreateReqVO.setLevel(1);

            //会员注册
            customerCreateReqVO.setSource(100);

            //注册会员是公司资源
            customerCreateReqVO.setResourceType(2);

            //线索客户
            customerCreateReqVO.setStatus(CustomerStatusEnum.CLUE.getValue());
            //更新 状态更新时间
            customerCreateReqVO.setStatusUpdateTime(new Date());


            //注册时自动创建
            customerCreateReqVO.setIsAuto(1);

            //客户类型 默认为发货人
            customerCreateReqVO.setType("0");

            //国家
            CountryDO countryDO = countryService.getCountry(areaCode);
            if (countryDO != null) {
                customerCreateReqVO.setCountry(countryDO.getId());
            }

            //添加联系人
            CustomerContactsCreateReqVO customerContactsCreateReqVO = new CustomerContactsCreateReqVO();
            customerContactsCreateReqVO.setName(newName);
            customerContactsCreateReqVO.setUserid(memberUserId);
            customerContactsCreateReqVO.setUsername(newName);
            //默认联系人
            customerContactsCreateReqVO.setIsDefault(1);
            customerContactsCreateReqVO.setAreaCode(areaCode);
            customerContactsCreateReqVO.setPhoneNew(phone);

            customerCreateReqVO.setCustomerContacts(Arrays.asList(customerContactsCreateReqVO));

            customerCreateReqVO.setCustomerOperateLogRemark("会员注册关联创建客户. 会员编号：" + memberCode + "; 电话：" + phone);

            // app创建
            customerCreateReqVO.setCreateFrom(CustomerCreateFromEnum.APP_REGISTER.getValue());

            customerService.createCustomer(customerCreateReqVO);
        }

    }

    public void associateCargoCustomerAuto(String destTitleEn,Integer transTypeId,String areaCode, String phone, String newName, String email) {
        //先查询此用户是否有关联老客户
        CustomerContactsDO customerContactsDO = customerContactsService.getUniqueOneAndValidate(areaCode, phone, null);
        if (customerContactsDO == null) {
            CustomerCreateReqVO customerCreateReqVO = new CustomerCreateReqVO();
            DictDataRespDTO transPortTypeDictDataRespDTO = DictFrameworkUtils.getDictDataFromCache("transport_type", String.valueOf(transTypeId));
            DictDataRespDTO customertTransPortTypeDictDataRespDTO = DictFrameworkUtils.parseDictDataFromCache("customer_transport_type", transPortTypeDictDataRespDTO.getLabel());
            if (customertTransPortTypeDictDataRespDTO!=null){
                customerCreateReqVO.setTransportType(customertTransPortTypeDictDataRespDTO.getValue());
            }
            DictDataRespDTO pickupPointDictDataRespDTO = DictFrameworkUtils.parseDictDataFromCache("pickup_points", destTitleEn);
            if (pickupPointDictDataRespDTO!=null){
                customerCreateReqVO.setPickupPoints(pickupPointDictDataRespDTO.getValue());
            }
            //初始化名字为手机号
            customerCreateReqVO.setName(newName);
            customerCreateReqVO.setNameEn(newName);
            //普通
            customerCreateReqVO.setLevel(1);

            //会员注册
            customerCreateReqVO.setSource(101);

            //注册会员是公司资源
            customerCreateReqVO.setResourceType(2);
            //业绩类型为老客户
            customerCreateReqVO.setIsNew(false);
            //设置手挡时间为当前时间
            customerCreateReqVO.setFirstDealTime(new Date());
            //成交客户
            customerCreateReqVO.setStatus(CustomerStatusEnum.DEAL.getValue());
            //更新 状态更新时间
            customerCreateReqVO.setStatusUpdateTime(new Date());
            //注册时自动创建
            customerCreateReqVO.setIsAuto(1);

            //客户类型 默认为收货人
            customerCreateReqVO.setType("1");

            //国家
            CountryDO countryDO = countryService.getCountry(areaCode);
            if (countryDO != null) {
                customerCreateReqVO.setCountry(countryDO.getId());
            }
            //添加联系人
            CustomerContactsCreateReqVO customerContactsCreateReqVO = new CustomerContactsCreateReqVO();
            customerContactsCreateReqVO.setName(newName);
            customerContactsCreateReqVO.setNameEn(newName);
            //默认联系人
            customerContactsCreateReqVO.setIsDefault(1);
            customerContactsCreateReqVO.setAreaCode(areaCode);
            customerContactsCreateReqVO.setPhoneNew(phone);
            customerContactsCreateReqVO.setEmail(email);
            customerCreateReqVO.setCustomerContacts(Arrays.asList(customerContactsCreateReqVO));

            customerCreateReqVO.setCustomerOperateLogRemark("控货收货人创建. 会员名称：" + newName + "; 电话：" + phone);

            // 控货收货人创建
            customerCreateReqVO.setCreateFrom(CustomerCreateFromEnum.CARGO_CONSIGNOR.getValue());

            customerService.createCustomer(customerCreateReqVO);
        }
    }


    @Override
    public void approvalCustomerDelay(String approveId, Integer result) {
        customerService.approvalCustomerDelay(approveId, result);
    }

    @Override
    public void approvalCustomerTransfer(String approveId, Integer result) {
        customerService.approvalCustomerTransfer(approveId, result);
    }

    @Override
    public void fillFirstDealTimeIfNull(Long customerId, Date firstDealTime) {
        customerService.update(new LambdaUpdateWrapper<CustomerDO>()
                .set(CustomerDO::getFirstDealTime, firstDealTime)
                .isNull(CustomerDO::getFirstDealTime)
                .eq(CustomerDO::getId, customerId));
    }
}
