package cn.iocoder.yudao.module.customer.service.customerApproval;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerApproval.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerApproval.CustomerApprovalDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import org.apache.ibatis.annotations.Param;

/**
 * 客户相关审核详情 Service 接口
 *
 * @author 系统管理员
 */
public interface CustomerApprovalService extends IService<CustomerApprovalDO> {

    /**
     * 创建客户相关审核详情
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCustomerApproval(@Valid CustomerApprovalCreateReqVO createReqVO);

    /**
     * 更新客户相关审核详情
     * @param updateReqVO 更新信息
     */
    void updateCustomerApproval(@Valid CustomerApprovalUpdateReqVO updateReqVO);

    /**
     * 删除客户相关审核详情
     * @param id 编号
     */
    void deleteCustomerApproval(Long id);

    /**
     * 获得客户相关审核详情
     * @param id 编号
     * @return 客户相关审核详情
     */
    CustomerApprovalDO getCustomerApproval(Long id);

    /**
     * 获得客户相关审核详情列表
     * @param ids 编号
     * @return 客户相关审核详情列表
     */
    List<CustomerApprovalDO> getCustomerApprovalList(Collection<Long> ids);

    /**
     * 获得客户相关审核详情分页
     * @param page 分页查询
     * @param query 查询
     * @return 客户相关审核详情分页
     */
    PageResult<CustomerApprovalDO> getCustomerApprovalPage(CustomerApprovalQueryVO query, PageVO page);

    /**
     * 获得客户相关审核详情列表, 用于 Excel 导出
     * @param query 查询
     * @return 客户相关审核详情列表
     */
    List<CustomerApprovalDO> getCustomerApprovalList(CustomerApprovalQueryVO query);

    /**
     * 通过客户id查询客户的移交审批
     *
     * @param customerId 客户id
     * @return
     */
    CustomerApprovalDO getCustomerHandoverApprovalByCustomerId(Long customerId);

    /**
     * 通过客户id查询所有类型的审批中的审批记录
     *
     * @param customerId
     * @return
     */
    List<CustomerApprovalDO> selectInApprovalByCustomerId(Long customerId);
}
