package cn.iocoder.yudao.module.customer.service.customerCompetitor;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.module.customer.vo.customerCompetitor.*;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerCompetitor.CustomerCompetitorDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

/**
 * 客户竞争对手 Service 接口
 *
 * @author yanghao
 */
public interface CustomerCompetitorService extends IService<CustomerCompetitorDO> {

    /**
     * 创建客户竞争对手
     * @param createReqVO 创建信息
     * @return 编号
     */
    CustomerCompetitorDO createCompetitor(@Valid CustomerCompetitorCreateReqVO createReqVO);

    /**
     * 更新客户竞争对手
     * @param updateReqVO 更新信息
     */
    void updateCompetitor(@Valid CustomerCompetitorUpdateReqVO updateReqVO);

    /**
     * 删除客户竞争对手
     * @param id 编号
     */
    void deleteCompetitor(Long id);

    /**
     * 获得客户竞争对手
     * @param id 编号
     * @return 客户竞争对手
     */
    CustomerCompetitorDO getCompetitor(Long id);

    /**
     * 获得客户竞争对手列表
     * @param ids 编号
     * @return 客户竞争对手列表
     */
    List<CustomerCompetitorDO> getCompetitorList(Collection<Long> ids);

    /**
     * 搜索
     * @param searchKey 关键字
     * @return
     */
    List<CustomerCompetitorDO> getCompetitorListBySearchKey(String searchKey);

    /**
     * 获得客户竞争对手分页
     * @param page 分页查询
     * @param query 查询
     * @return 客户竞争对手分页
     */
    PageResult<CustomerCompetitorDO> getCompetitorPage(CustomerCompetitorQueryVO query, PageVO page);

    /**
     * 获得客户竞争对手列表, 用于 Excel 导出
     * @param query 查询
     * @return 客户竞争对手列表
     */
    List<CustomerCompetitorDO> getCompetitorList(CustomerCompetitorQueryVO query);
}
