package cn.iocoder.yudao.module.customer.service.customerContacts;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.validation.PhoneUtil;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.customer.convert.customerContacts.CustomerContactsConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customerContacts.CustomerContactsMapper;
import cn.iocoder.yudao.module.customer.dto.CustomerContactsDto;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsCreateReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.customerContacts.CustomerContactsUpdateReqVO;
import cn.iocoder.yudao.module.ecw.service.paramValid.ParamValidatorService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_CONTACTS_NOT_EXISTS;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.CUSTOMER_CONTACTS_PHONE_FORMAT_ERROR;

/**
 * 客户联系人 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class CustomerContactsServiceImpl extends AbstractService<CustomerContactsMapper, CustomerContactsDO> implements CustomerContactsService {

    @Resource
    private CustomerContactsMapper customerContactsMapper;
    @Resource
    private ParamValidatorService paramValidatorService;

    @Override
    public Long createCustomerContacts(CustomerContactsCreateReqVO createReqVO) {
        // 手机号校验
        try {
            String mobileCode = createReqVO.getAreaCode() + StrUtil.COLON + createReqVO.getPhoneNew();
            paramValidatorService.validatorMobile(mobileCode);
//            String phone = PhoneUtil.formatPhone(mobileCode);
//            if (StringUtils.isNotBlank(phone)) {
//                createReqVO.setPhoneNew(phone);
//            }
        } catch (Exception e) {
            throw exception(CUSTOMER_CONTACTS_PHONE_FORMAT_ERROR);
        }
        // 插入
        CustomerContactsDO customerContacts = CustomerContactsConvert.INSTANCE.convert(createReqVO);
        customerContactsMapper.insert(customerContacts);
        // 返回
        return customerContacts.getId();
    }

    @Override
    public void updateCustomerContacts(CustomerContactsUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateCustomerContactsExists(updateReqVO.getId());
        // 手机号校验
        try {
            String mobileCode = updateReqVO.getAreaCode() + StrUtil.COLON + updateReqVO.getPhoneNew();
            paramValidatorService.validatorMobile(mobileCode);
//            String phone = PhoneUtil.formatPhone(mobileCode);
//            if (StringUtils.isNotBlank(phone)) {
//                updateReqVO.setPhoneNew(phone);
//            }
        } catch (Exception e) {
            throw exception(CUSTOMER_CONTACTS_PHONE_FORMAT_ERROR);
        }
        // 更新
        CustomerContactsDO updateObj = CustomerContactsConvert.INSTANCE.convert(updateReqVO);
        customerContactsMapper.updateById(updateObj);
    }

    @Override
    public void deleteCustomerContacts(Long id) {
        // 校验存在
        this.validateCustomerContactsExists(id);
        // 删除
        customerContactsMapper.deleteById(id);
    }

    private void validateCustomerContactsExists(Long id) {
        if (customerContactsMapper.selectById(id) == null) {
            throw exception(CUSTOMER_CONTACTS_NOT_EXISTS);
        }
    }

    @Override
    public CustomerContactsDO getCustomerContacts(Long id) {
        return customerContactsMapper.selectById(id);
    }

    @Override
    public List<CustomerContactsDO> getCustomerContactsListByCustomerId(Collection<Long> ids) {
        return customerContactsMapper.selectBatchIds(ids);
    }

    @Override
    public List<CustomerContactsDO> getByPhone(String areaCode, String phone) {
        return customerContactsMapper.selectList(new LambdaQueryWrapperX<CustomerContactsDO>()
                .eqIfPresent(CustomerContactsDO::getAreaCode, areaCode)
                .eqIfPresent(CustomerContactsDO::getPhoneNew, phone));
    }

    @Override
    public List<CustomerContactsDO> getByMemberUserId(Long memberUserId) {
        return customerContactsMapper.selectList(new LambdaQueryWrapperX<CustomerContactsDO>()
                .eqIfPresent(CustomerContactsDO::getUserid, memberUserId));
    }

    @Override
    public List<CustomerContactsDO> getCustomerContactsListByCustomerId(Long customerId) {
        return customerContactsMapper.selectList(new LambdaQueryWrapperX<CustomerContactsDO>().eqIfPresent(CustomerContactsDO::getCustomerId, customerId));
    }

    @Override
    public PageResult<CustomerContactsDO> getCustomerContactsPage(CustomerContactsPageReqVO pageReqVO) {
        return customerContactsMapper.selectPage(pageReqVO);
    }

    @Override
    public List<CustomerContactsDO> getCustomerContactsListByCustomerId(CustomerContactsExportReqVO exportReqVO) {
        return customerContactsMapper.selectList(exportReqVO);
    }

    @Override
    public CustomerContactsDO getDefaultContacts(Long customerId) {
        return customerContactsMapper.selectOne(new LambdaQueryWrapper<CustomerContactsDO>()
                .eq(CustomerContactsDO::getIsDefault, 1)
                .eq(CustomerContactsDO::getCustomerId, customerId)
                .last("limit 1"));
    }

    @Override
    public PageResult<CustomerContactsDto> selectBySearchKey(String searchKey, String phone, String type, Collection<Long> ids, Long customerId, PageParam page) {
        IPage<CustomerContactsDto> mpPage = MyBatisUtils.buildPage(page);
        customerContactsMapper.selectBySearchKey(mpPage, searchKey, phone,
                type, ids, customerId);

        return PageResult.of(mpPage);
    }

    @Override
    public PageResult<CustomerContactsDto> selectBySearchKey2(
            String searchKey,
            String phone,
            String type,
            Collection<Long> ids,
            Long customerId,
            PageParam page,
            Long UserId) {
        IPage<CustomerContactsDto> mpPage = MyBatisUtils.buildPage(page);
        customerContactsMapper.selectBySearchKey2(mpPage, searchKey, phone,
                type, ids, customerId,UserId);

        return PageResult.of(mpPage);
    }

    @Override
    @Async
    public void asyncUpdateEmail(Long customerContactsId, String email) {
        if (StringUtils.isNotBlank(email)) {
            CustomerContactsDO customerDO = new CustomerContactsDO();
            customerDO.setId(customerContactsId);
            customerDO.setEmail(email);
            this.updateById(customerDO);
        }
    }

    @Override
    public CustomerContactsDO getUniqueOneAndValidate(String areaCode, String phone, Long excludeCustomerId) {

        // 手机号校验
        String newPhone = null;
        try {
            String mobileCode = areaCode + StrUtil.COLON + phone;
            paramValidatorService.validatorMobile(mobileCode);
            newPhone = PhoneUtil.formatPhone(mobileCode);
        } catch (Exception e) {
            throw exception(CUSTOMER_CONTACTS_PHONE_FORMAT_ERROR);
        }

        String newPhoneFinal = newPhone;
        CustomerContactsDO customerContactsDO = this.getOne(new LambdaQueryWrapperX<CustomerContactsDO>()
                .eq(CustomerContactsDO::getAreaCode, areaCode)
                .eq(CustomerContactsDO::getDeleted, false)
                // 判断相等: 1、原输入 或2、0开头去0  或3、非0开头加0  或4、格式化后的值
                .and(i -> i.eq(CustomerContactsDO::getPhoneNew, phone)
                        .or()
                        .eq(CustomerContactsDO::getPhoneNew, phone.startsWith("0") ?  phone.substring(1) : ("0" + phone))
                        .or(StringUtils.isNotEmpty(newPhoneFinal))
                        .eq(StringUtils.isNotEmpty(newPhoneFinal), CustomerContactsDO::getPhoneNew, newPhoneFinal)
                ).apply(" customer_id is not null ")
                .apply(excludeCustomerId != null, " customer_id != {0} ", excludeCustomerId)
                .orderByDesc(CustomerContactsDO::getId).last("limit 1"), false);
        // if (customerContactsDO != null) {
        //     throw exception(ErrorCodeConstants.CUSTOMER_CONTACTS_ALREADY_BIND, phone);
        // }

        return customerContactsDO;
    }
}
