package cn.iocoder.yudao.module.customer.service.customerFollowup;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.iocoder.yudao.framework.apollo.core.event.Order.OrderNumberLogEvent;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.code.CodeUtils;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.customer.convert.customerFollowup.CustomerFollowupConvert;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.CustomerDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerContacts.CustomerContactsDO;
import cn.iocoder.yudao.module.customer.dal.dataobject.customerFollowup.CustomerFollowupDO;
import cn.iocoder.yudao.module.customer.dal.mysql.customer.CustomerMapper;
import cn.iocoder.yudao.module.customer.dal.mysql.customerContacts.CustomerContactsMapper;
import cn.iocoder.yudao.module.customer.dal.mysql.customerFollowup.CustomerFollowupMapper;
import cn.iocoder.yudao.module.customer.service.customerOperateLog.CustomerOperateLogService;
import cn.iocoder.yudao.module.customer.vo.customerFollowup.*;
import cn.iocoder.yudao.module.customer.vo.customerOperateLog.CustomerOperateLogCreateReqVO;
import cn.iocoder.yudao.module.ecw.enums.CustomerFollowupStatusEnum;
import cn.iocoder.yudao.module.ecw.enums.CustomerOperateTypeEnum;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationContext;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;
import static cn.iocoder.yudao.module.customer.enums.ErrorCodeConstants.*;

/**
 * 客户跟进 Service 实现类
 *
 * @author yanghao
 */
@Service
@Validated
public class CustomerFollowupServiceImpl extends AbstractService<CustomerFollowupMapper, CustomerFollowupDO> implements CustomerFollowupService {

    @Resource
    private CustomerFollowupMapper followupMapper;

    @Resource
    private AdminUserApi adminUserApi;
    @Resource
    private CustomerOperateLogService customerOperateLogService;

    @Resource
    private CustomerMapper customerMapper;

    @Resource
    private CustomerContactsMapper customerContactsMapper;

    // 注入applicationContext
    @Resource
    private ApplicationContext applicationContext;


    private static final String FOLLOWUP_NEW_NUMBER = "followup:new:number";

    /**
     * 跟进单编号生成方式：GJ+年份+六位数字，按年份累计
     * @return
     */
    @Override
    public CustomerFollowupNumberBackVO generateFollowupNumber() {
        // 跟进单号生成
        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(6);
        event.setNumberKey(FOLLOWUP_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newFollowupNumber = CodeUtils.getOddFollowupNumbers(event.getNumber());
        CustomerFollowupNumberBackVO customerFollowupNumberBackVO =
                new CustomerFollowupNumberBackVO()
                        .setNumber(newFollowupNumber)
                        .setNumberId(event.getId())
                        .setNumberVersion(event.getVersion());

        return  customerFollowupNumberBackVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long createFollowup(CustomerFollowupCreateReqVO createReqVO) {

        // 更新编号，内部会判断冲突
        //OrderNumberLogListener
        OrderNumberLogEvent event = new OrderNumberLogEvent();
        event.setType(6);
        event.setNumberKey(FOLLOWUP_NEW_NUMBER);
        applicationContext.publishEvent(event);
        String newNumber = CodeUtils.getOddFollowupNumbers(event.getNumber());
        // 更新跟进记录编号最新缓存
        // 待将最新单号持久化
        event.setNewNumber(newNumber);
        applicationContext.publishEvent(event);
        // 插入
        CustomerFollowupDO followup = CustomerFollowupConvert.INSTANCE.convert(createReqVO);
        if (Objects.isNull(followup.getFollowType())){
            throw exception(FOLLOWUP_TYPE_NOT_NULL);
        }
        if (followup.getFollowType() == 2 && Objects.isNull(followup.getOfferId())){
            throw exception(BUSINESS_OPPORTUNITY_FOLLOWUP_OFFER_NOT_NULL);
        }
        // 设置联系人名称
        setContactName(followup);
        followup.setNumber(newNumber);
        if (Objects.nonNull(followup.getFollowUserId())){
            // 获取客户经理的部门id
            followup.setDeptId(adminUserApi.getUserDeptId(followup.getFollowUserId()));
        }
        // 当跟进为提交结果时，除附件和上级跟进单非必填，其他参数都时必填
        if (followup.getStatus() == 1){
            validateFollowupMustParams(followup);
        }
        followupMapper.insert(followup);

        Long customerId = createReqVO.getCustomerId();
        if (customerId != null) {
            CustomerDO customer = customerMapper.selectById(customerId);
            if (customer != null) {

                // 插入日志
                LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                //保存客户捞取记录
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(loginUser == null ? null : loginUser.getId())
                        .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setNewCustomerService(customer.getCustomerService())
                        .setOldCustomerService(customer.getCustomerService())
                        .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_CREATE.getValue())
                        .setRemark("新增跟进纪录" + (CustomerFollowupStatusEnum.UnCommited.getValue().equals(createReqVO.getStatus()) ? "" : "，并提交"));
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        }

        // 返回
        return followup.getId();
    }

    private void validateFollowupMustParams(CustomerFollowupDO followup) {
        if (Objects.isNull(followup.getCustomerId())){
            throw exception(CUSTOMER_NOT_NULL);
        }
        if (Objects.isNull(followup.getContactId())){
            throw exception(CUSTOMER_CONTACT_NOT_NULL);
        }

        if (Objects.isNull(followup.getFollowTime())){
            throw exception(FOLLOWUP_TIME_NOT_NULL);
        }
        if (Objects.isNull(followup.getFollowMethod())){
            throw exception(FOLLOWUP_METHOD_NOT_NULL);
        }
        if (Objects.isNull(followup.getFollowUserId())){
            throw exception(CUSTOMER_SERVICE_NOT_NULL);
        }
        if (Objects.isNull(followup.getSaleStage())){
            throw exception(SALE_STAGE_NOT_NULL);
        }
//        if (Objects.isNull(followup.getResultType())){
//            throw exception(FOLLOWUP_RESULT_NOT_NULL);
//        }
        if (StringUtils.isBlank(followup.getPurpose())){
            throw exception(FOLLOWUP_PURPOSE_NOT_NULL);
        }
        if (StringUtils.isBlank(followup.getFeedback())){
            throw exception(FOLLOWUP_FEEFBACK_NOT_NULL);
        }
        if (Objects.isNull(followup.getNextTime())){
            throw exception(NEXT_FOLLOWUP_TIME_NOT_NULL);
        }
        if (StringUtils.isBlank(followup.getNextPlan())){
            throw exception(NEXT_FOLLOWUP_PLAN_NOT_NULL);
        }
    }

    private void setContactName(CustomerFollowupDO followup) {
        // 设置联系人名称
        Long contactId = followup.getContactId();
        if (contactId != null) {
            CustomerContactsDO customerContactsDO = customerContactsMapper.selectById(contactId);
            if (customerContactsDO != null) {
                followup.setContactName(I18nMessage.getLang() == 0 ? customerContactsDO.getName() : customerContactsDO.getNameEn());
            }
        }
    }

    @Override
    public void updateFollowup(CustomerFollowupUpdateReqVO updateReqVO) {

        // 校验存在
        Long id = updateReqVO.getId();
        CustomerFollowupDO customerFollowupDO = followupMapper.selectById(id);
        if (customerFollowupDO == null) {
            throw exception(FOLLOWUP_NOT_EXISTS);
        }
        if (customerFollowupDO.getStatus().equals(CustomerFollowupStatusEnum.Commited.getValue())) {
            throw exception(FOLLOWUP_ALREADY_SUBMITTED);
        }
        // 更新
        CustomerFollowupDO updateObj = CustomerFollowupConvert.INSTANCE.convert(updateReqVO);
        if (Objects.isNull(updateObj.getFollowType())){
            throw exception(FOLLOWUP_TYPE_NOT_NULL);
        }
        if (updateObj.getFollowType() == 2 && Objects.isNull(updateObj.getOfferId())){
            throw exception(BUSINESS_OPPORTUNITY_FOLLOWUP_OFFER_NOT_NULL);
        }
        // 设置联系人名称
        setContactName(updateObj);
        if (Objects.nonNull(updateObj.getFollowUserId())){
            // 获取客户经理的部门id
            updateObj.setDeptId(adminUserApi.getUserDeptId(updateObj.getFollowUserId()));
        }
        // 当跟进为提交结果时，除附件和上级跟进单非必填，其他参数都时必填
        if (updateObj.getStatus() == 1){
            validateFollowupMustParams(updateObj);
        }
        followupMapper.updateById(updateObj);

        Long customerId = updateReqVO.getCustomerId();
        if (customerId != null) {
            CustomerDO customer = customerMapper.selectById(customerId);
            if (customer != null) {

                // 插入日志
                LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                //保存客户捞取记录
                CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                        .setOperator(loginUser == null ? null : loginUser.getId())
                        .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                        .setCustomerId(customer.getId())
                        .setNumber(customer.getNumber())
                        .setName(customer.getName())
                        .setNewCustomerService(customer.getCustomerService())
                        .setOldCustomerService(customer.getCustomerService())
                        .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                        .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_UPDATE.getValue())
                        .setRemark("更新跟进纪录" + (CustomerFollowupStatusEnum.UnCommited.getValue().equals(updateReqVO.getStatus()) ? "" : "，并提交"));
                customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
            }
        }
    }

    @Override
    public void updateFollowupStatus(CustomerFollowupUpdateStatusReqVO updateStatusReqVO) {
        List<Long> ids = updateStatusReqVO.getIds();

        for (Long id : ids) {
            CustomerFollowupDO customerFollowupDO = followupMapper.selectById(id);
            if (customerFollowupDO == null) {
                throw exception(FOLLOWUP_NOT_EXISTS);
            }
//
//            if (CustomerFollowupStatusEnum.Commited.getValue().equals(customerFollowupDO.getStatus())) {
//                throw exception(FOLLOWUP_ALREADY_SUBMITTED);
//            }
        }
        Date now = DateUtil.date();
        Long userId = SecurityFrameworkUtils.getLoginUser().getId();
        for (Long id : ids) {
            CustomerFollowupDO customerFollowupDO = followupMapper.selectById(id);

            Integer statusOld = customerFollowupDO.getStatus();

            customerFollowupDO.setStatus(updateStatusReqVO.getStatus());
            customerFollowupDO.setUpdater(String.valueOf(userId));
            customerFollowupDO.setUpdateTime(now);
            followupMapper.updateById(customerFollowupDO);


            Long customerId = customerFollowupDO.getCustomerId();
            if (customerId != null) {
                CustomerDO customer = customerMapper.selectById(customerId);
                if (customer != null) {
                    // 插入日志
                    LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
                    //保存客户捞取记录
                    CustomerOperateLogCreateReqVO customerOperateLogCreateReqVO = new CustomerOperateLogCreateReqVO()
                            .setOperator(loginUser == null ? null : loginUser.getId())
                            .setOperatorName(loginUser == null ? null : loginUser.getNickname())
                            .setCustomerId(customer.getId())
                            .setNumber(customer.getNumber())
                            .setName(customer.getName())
                            .setNewCustomerService(customer.getCustomerService())
                            .setOldCustomerService(customer.getCustomerService())
                            .setOldEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                            .setNewEstimateEnterOpenSeaTime(customer.getEstimateEnterOpenSeaTime())
                            .setOperateType(CustomerOperateTypeEnum.FOLLOWUP_UPDATE_STATUS.getValue())
                            .setRemark(String.format("由【%s】修改为【%s】", statusOld == 1 ? "已提交" : "未提交", updateStatusReqVO.getStatus() == 1 ? "已提交" : "未提交"));
                    customerOperateLogService.createOperateLog(customerOperateLogCreateReqVO);
                }
            }
        }
    }

    @Override
    public void deleteFollowup(Long id) {
        // 校验存在
        this.validateFollowupExists(id);
        // 删除
        followupMapper.deleteById(id);
    }

    private void validateFollowupExists(Long id) {
        if (followupMapper.selectById(id) == null) {
            throw exception(FOLLOWUP_NOT_EXISTS);
        }
    }

    @Override
    public CustomerFollowupDO getFollowup(Long id) {
        return followupMapper.selectById(id);
    }

    @Override
    public List<CustomerFollowupDO> getFollowupList(Collection<Long> ids) {
        return followupMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<CustomerFollowupDO> getFollowupPage(CustomerFollowupQueryVO query, PageVO page) {

        page.setPage(query.getPageNo());
        page.setRows(query.getPageSize());

        IPage<CustomerDO> mpPage = MyBatisUtils.buildPage(page);
        log.warn(I18nMessage.getLang().toString());
        int start = (page.getPage() - 1) * page.getRows();
        int size = page.getRows();
        List<CustomerFollowupDO> list = followupMapper.selectPage1(start, size, query);
        long total = followupMapper.selectCount1(query);
        return new PageResult<>(list, total,
                mpPage.getSize(), page.getPage(),
                (total + mpPage.getSize() - 1) / mpPage.getSize());
    }

    @Override
    public List<CustomerFollowupDO> getFollowupList(CustomerFollowupQueryVO query) {
        return followupMapper.selectPage1(null, null, query);
    }
}
