package cn.iocoder.yudao.module.customer.vo.customer.vo;

import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 客户 Base VO，提供给添加、修改、详细的子 VO 使用
 * 如果子 VO 存在差异的字段，请不要添加到这里，影响 Swagger 文档生成
 */
@Data
public class CustomerBaseVO {

    @ApiModelProperty(value = "客户编号")
    private String number;

    @ApiModelProperty(value = "客户名称", required = true)
    @NotNull(message = "客户名称不能为空")
    private String name;

    /**
     * 客户英文名称
     */
    @ApiModelProperty(value = "客户英文名称")
    private String nameEn;

    @ApiModelProperty(value = "客户等级 参见字典 customer_level")
    private Integer level = 1;

    @ApiModelProperty(value = "客户信用等级 参见表ecw_customer_credit")
    private Integer creditLevel;

    @ApiModelProperty(value = "国家")
    private Long country;

    @ApiModelProperty(value = "跟进客服")
    private Long customerService;

    @ApiModelProperty(value = "客户状态 参见字典 customer_status")
    private Integer status;

    @JsonIgnore
    private Date statusUpdateTime;

    @ApiModelProperty(value = "部门")
    private String department;


    /**
     * vip等级分
     */
    @ApiModelProperty(value = "vip等级分")
    private Long vipLevelScore;


    /**
     * vip等级
     */
    @ApiModelProperty(value = "vip等级-中文")
    private String vipLevelNameZh;

    /**
     * vip等级
     */
    @ApiModelProperty(value = "vip等级-英文")
    private String vipLevelNameEn;

    /**
     * 信用等级分
     */
    @ApiModelProperty(value = "信用等级分")
    private Long creditLevelScore;


    /**
     * 信用等级
     */
    @ApiModelProperty(value = "信用等级-中文")
    private String creditLevelNameZh;

    /**
     * 信用等级
     */
    @ApiModelProperty(value = "信用等级-英文")
    private String creditLevelNameEn;

    @ApiModelProperty(value = "跟进客服名字-客户经理")
    private String customerServiceName;


    @ApiModelProperty(value = "身份证名称")
    private String carName;

    @ApiModelProperty(value = "身份证号码")
    private String carNo;



    /**
     * 收货人首次分配的发货人的客户经理
     */
    @ApiModelProperty(value = "收货人首次分配的发货人的客户经理")
    private Long consigneeFirstCustomerService;

    /**
     * 是否新客户
     */
    @ApiModelProperty(value = "是否新客户")
    private Boolean isNew;

    /**
     * 付款人姓名
     */
    @ApiModelProperty(value = "付款人姓名")
    private String payerName;

    @ApiModelProperty(value = "主营产品类别分组")
    private List<CustomerProductTypeGroupVO> products;


}
